@@ -20,10 +20,12 @@
 
 ***************************************************************************/
 
-//     zfuncs.h   version  v.4.7
+//     zfuncs.h   version  v.4.9
 
 #include <sys/time.h>
 #include <sys/stat.h>
+#include <sys/resource.h>
+#include <sys/file.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -36,7 +38,6 @@
 #include <signal.h>
 #include <execinfo.h>
 #include <locale.h>
-#include <sys/resource.h>
 
 #define  int8   char                                                       //  number types
 #define  int16  short
@@ -83,6 +84,8 @@ double jobtime();                                                          //
 int  parseprocfile(cchar *pfile, cchar *pname, double *value, ...);        //  get data from /proc file
 int  parseprocrec(char *prec, int field, double *value, ...);              //  get data from /proc file record
 void zsleep(double dsecs);                                                 //  sleep specified seconds
+int global_lock(cchar *lockname);                                          //  obtain exclusive lock, multi-process
+int global_unlock(int fd);                                                 //  release the lock
 
 void start_detached_thread(void * tfunc(void *), void * arg);              //  start detached thread function
 void synch_threads(int NT = 0);                                            //  synchronize NT threads
@@ -266,9 +269,6 @@ void ZTX_translation_start();                                              //  s
 
 #define  zdcbmax 100                                                       //  max. combo box drop-down list
 
-void zlockInit();                                                          //  lock GTK for threaded applications
-void zlock();  
-void zunlock();
 void zmainloop(int skip = 0);                                              //  do main loop, process menu events
 
 //  text window print and read utilities
@@ -290,12 +290,19 @@ void wprintp(GtkWidget *Win);                                              //  p
 #define G_SIGNAL(window,event,func,arg) g_signal_connect(G_OBJECT(window),event,G_CALLBACK(func),(void *) arg);
 
 typedef void mtFunc(GtkWidget *, cchar *mname);                                  //  menu or button response function
+
 GtkWidget * create_menubar(GtkWidget *vbox);                                     //  create menubar in packing box
 GtkWidget * add_menubar_item(GtkWidget *mbar, cchar *mname, mtFunc func = 0);    //  add menu item to menubar 
 GtkWidget * add_submenu_item(GtkWidget *mitem, cchar *label, mtFunc func = 0);   //  add submenu item to menu item 
+
+GtkWidget * create_popmenu();                                                    //  create an empty popup menu
+GtkWidget * add_popmenu_item(GtkWidget *popmenu, cchar *mname, mtFunc func);     //  add menu item to popup menu
+void popup_menu(GtkWidget *popmenu);                                             //  pop-up menu at current mouse position
+
 GtkWidget * create_toolbar(GtkWidget *vbox, int iconsize = 24, int vert = 0);    //  horiz. toolbar in packing box (vert.)
 GtkWidget * add_toolbar_button(GtkWidget *tbar, cchar *label, cchar *tip,        //  add button with stock (gtk-quit) 
                                  cchar *icon, mtFunc func);                      //    or custom icon (iconfile.png)
+
 GtkWidget * create_stbar(GtkWidget *vbox);                                       //  create status bar in packing box
 int stbar_message(GtkWidget *stbar, cchar *message);                             //  display message in status bar
 
@@ -399,6 +406,7 @@ int zmessageYN(GtkWidget *parent, cchar *pMess, ... );                     //  d
 void zmessage_help(GtkWidget *parent, cchar *topic, cchar *mess, ...);     //  message box, [help] > user guide topic
 zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *mess, ...);    //  display message until timeout or cancel
 char * zdialog_text(GtkWidget *parent, cchar * title, cchar * initext);    //  get short text input from user
+int zdialog_choose(cchar *title, GtkWidget *parent, cchar *mess, ...);     //  show message and return choice button
 
 //  file chooser dialogs for 1 file or multiple files
 char  * zgetfile1(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
@@ -423,7 +431,7 @@ void print_image_file(cchar *imagefile);                                   //  p
 GdkPixbuf * gdk_pixbuf_rotate(GdkPixbuf *, double deg, int alfa = 0);      //  rotate pixbuf through any angle
 
 char * image_gallery(cchar *filez, cchar *action, int Nth = 0,             //  display image gallery window, navigate, 
-                      void ufunc(int Nth) = 0, GtkWidget *parent = 0);     //    do callback for clicked thumbnails
+       void ufunc(int Nth, int butt) = 0, GtkWidget *parent = 0);          //    do callback for clicked thumbnails
 
 int image_gallery_position(cchar *file, int Nth);                          //  get relative position of file in gallery
 int image_file_type(cchar *file);                                          //  determine if directory or image file type
